==> htnews-0.6.2.tar.gz: htnews-0.6.2/util.cc <==
#include <stdio.h>
#include <iostream.h>
#include <ctype.h>
#include <string>
#include <stdlib.h>
#include <fstream>
#include "htnews.h"

extern string SENDER;

string getval(const string& buf, const string& param)
{
        string::size_type pos;
        string tmp(""), out("");
        int stop=0;
        int yes=1;
        int no=0;
        int len;
        pos = buf.find(param);
        if( pos != string::npos )
                tmp.assign(buf, pos + param.length());
        else
                return "";
        len = tmp.length();
        for(int i=0;i<len;i++)
        {
                if((tmp[i] != ' ') || (stop != no))
                {
                        stop = yes;
                        out += tmp[i];
                }
        }
        return out;
}

string replace(const string& line)
{
        int i, u;
        string temp("");
        i = line.length();
        for(u=0;u<i;u++)
        {
                if(line[u] == '"')
                        temp += "\\\"";
                else
                        temp += line[u];
        }
        return temp;
}


void error()
{
        cerr << "usage:    htnews [-c configfile]\n"
             << "If -c is used, this configfile will be parsed, otherwise\n"
             << "it searches for a configfile named /etc/htnews.conf\n"
             << "If the file does not exist, the program exits\n";
        exit(1);
}

void err(string Tmp_File, string MSG, string Admin)
{
        ofstream Tmp("/tmp/htnews.err");
        Tmp << MSG;
        if(Tmp_File == "Standard")
        {
                Tmp << "This seems to be a configuration error.\n";
        }
        else if(Tmp_File == "refresh")
                Tmp << "\n";
        else
        {
                Tmp << "\n\n--- Original Message follows ---\n\n";
                Tmp.close();
                string CPY("cat ");
                CPY.append(Tmp_File);
                CPY.append(" >> ");
                CPY.append("/tmp/htnews.err");
                system(CPY.c_str());
        }
        Tmp.close();
        string CMD("");
        CMD.append("(echo \"From: ");
        CMD.append(SENDER);
        CMD.append("\"; echo \"To: ");
        CMD.append(Admin);
        CMD.append("\"; echo \"Subject: ");
        if(Tmp_File != "refresh")
                CMD.append("Error while trying to access htnews robot.\"; echo; echo \"");
        else
                CMD.append("Refresh done.\"; echo; echo \"");
        CMD.append("`cat /tmp/htnews.err`");
        CMD.append("\n");
        CMD.append("\") | " + string(_SENDMAIL));
        system(CMD.c_str());
        system("rm /tmp/htnews.*");
        exit(0);
}


int hex2int(const string& line)
{
        int value=0;
        char input[80]; // ok, because line is limited to 76 chars!
	strcpy(input, line.c_str());
        if(isxdigit(*input))
        {
                sscanf(input, "%02X", &value);
        }
	return value;
}

string quoted2html(const string& quoted)
{
	/*
	 * This function converts quoted-printable chars to html
	 *
	 * i.e.:
	 * in = "<body bgcolor=3D\"#ffffff\">\n<title>T=4Ften=3D=3F=\n</title>\n";
	 * would be:
	 * "<body bgcolor="#ffffff">\n<title>T&#79;ten=&#63;</title>"
	 * 
	 * everything after a "=" will be converted with the function hex2int to an 
	 * int, which will be passed into html-code like "&#int;" (where int is the number)
	 * If the "=" is the last char on a line (directly followed by '\n', then
	 * it is a "soft-linebreak" and will not be transtated to a linebreak.
	 * If a "=3D" occurs, which is int 61, then a "=" will be translated.
	 *
	 * If a "=20" occurs, a ' ' (blank) will be translated. If after the blank a
	 * newline follows, this newline will be removed, since the output is html this
	 * does not matter. And there will no breaked tags be anymore. Example:
	 * "<font size=3D2=20
	 * "face=3Darial>"
	 * will be:
	 * "<font size=2 face=arial>"
	 */
	string New(""), tocheck("");
	char buf[5];
	int i, len;
	len = quoted.length();
	for(i=0;i<len;i++)
	{
		if(quoted[i] == '=')
		{
			if(quoted[i+1] != '\n')
			{
				tocheck = "";
				tocheck += quoted[i+1];
				tocheck += quoted[i+2];
				if(hex2int(tocheck) == 61)
				{
					New += '=';
					i += 2;
				}
				else if(hex2int(tocheck) == 32)
				{
					New += ' ';
					if(quoted[i+3] == '\n')
						i += 3;
					else
						i += 2;	
				}
				else
				{
					sprintf(buf, "%d", hex2int(tocheck));
					New += "&#";
					New.append(buf);
					New += ";";
					i += 2;
				}
			}
			else
			{
				// remove the '\n'
				i++;
			}
		}
		else
			New += quoted[i];
	}
	return New;
}

==> htnews-0.6.3.tar.gz: htnews-0.6.3/util.cc <==
#include <stdio.h>
#include <iostream.h>
#include <ctype.h>
#include <string>
#include <stdlib.h>
#include <fstream>
#include "htnews.h"

extern string SENDER;

string getval(const string& buf, const string& param)
{
        string::size_type pos;
        string tmp(""), out("");
        int stop=0;
        int yes=1;
        int no=0;
        int len;
        pos = buf.find(param);
        if( pos != string::npos )
                tmp.assign(buf, pos + param.length());
        else
                return "";
        len = tmp.length();
        for(int i=0;i<len;i++)
        {
                if((tmp[i] != ' ') || (stop != no))
                {
                        stop = yes;
                        out += tmp[i];
                }
        }
        return out;
}

string replace(const string& line)
{
        int i, u;
        string temp("");
        i = line.length();
        for(u=0;u<i;u++)
        {
                if(line[u] == '"')
                        temp += "\\\"";
                else
                        temp += line[u];
        }
        return temp;
}


void error()
{
        cerr << "usage:    htnews [-c configfile]\n"
             << "If -c is used, this configfile will be parsed, otherwise\n"
             << "it searches for a configfile named /etc/htnews.conf\n"
             << "If the file does not exist, the program exits\n";
        exit(1);
}

void err(string Tmp_File, string MSG, string Admin)
{
        ofstream Tmp("/tmp/htnews.err");
        Tmp << MSG;
        if(Tmp_File == "Standard")
        {
                Tmp << "This seems to be a configuration error.\n";
        }
        else if(Tmp_File == "refresh")
                Tmp << "\n";
        else
        {
                Tmp << "\n\n--- Original Message follows ---\n\n";
                Tmp.close();
                string CPY("cat ");
                CPY.append(Tmp_File);
                CPY.append(" >> ");
                CPY.append("/tmp/htnews.err");
                system(CPY.c_str());
        }
        Tmp.close();
        string CMD("");
        CMD.append("(echo \"From: ");
        CMD.append(SENDER);
        CMD.append("\"; echo \"To: ");
        CMD.append(Admin);
        CMD.append("\"; echo \"Subject: ");
        if(Tmp_File != "refresh")
                CMD.append("Error while trying to access htnews robot.\"; echo; echo \"");
        else
                CMD.append("Refresh done.\"; echo; echo \"");
        CMD.append("`cat /tmp/htnews.err`");
        CMD.append("\n");
        CMD.append("\") | " + string(_SENDMAIL) + " -t");
        system(CMD.c_str());
        system("rm /tmp/htnews.*");
        exit(0);
}


int hex2int(const string& line)
{
        int value=0;
        char input[80]; // ok, because line is limited to 76 chars!
	strcpy(input, line.c_str());
        if(isxdigit(*input))
        {
                sscanf(input, "%02X", &value);
        }
	return value;
}

string quoted2html(const string& quoted)
{
	/*
	 * This function converts quoted-printable chars to html
	 *
	 * i.e.:
	 * in = "<body bgcolor=3D\"#ffffff\">\n<title>T=4Ften=3D=3F=\n</title>\n";
	 * would be:
	 * "<body bgcolor="#ffffff">\n<title>T&#79;ten=&#63;</title>"
	 * 
	 * everything after a "=" will be converted with the function hex2int to an 
	 * int, which will be passed into html-code like "&#int;" (where int is the number)
	 * If the "=" is the last char on a line (directly followed by '\n', then
	 * it is a "soft-linebreak" and will not be transtated to a linebreak.
	 * If a "=3D" occurs, which is int 61, then a "=" will be translated.
	 *
	 * If a "=20" occurs, a ' ' (blank) will be translated. If after the blank a
	 * newline follows, this newline will be removed, since the output is html this
	 * does not matter. And there will no breaked tags be anymore. Example:
	 * "<font size=3D2=20
	 * "face=3Darial>"
	 * will be:
	 * "<font size=2 face=arial>"
	 */
	string New(""), tocheck("");
	char buf[5];
	int i, len;
	len = quoted.length();
	for(i=0;i<len;i++)
	{
		if(quoted[i] == '=')
		{
			if(quoted[i+1] != '\n')
			{
				tocheck = "";
				tocheck += quoted[i+1];
				tocheck += quoted[i+2];
				if(hex2int(tocheck) == 61)
				{
					New += '=';
					i += 2;
				}
				else if(hex2int(tocheck) == 32)
				{
					New += ' ';
					if(quoted[i+3] == '\n')
						i += 3;
					else
						i += 2;	
				}
				else
				{
					sprintf(buf, "%d", hex2int(tocheck));
					New += "&#";
					New.append(buf);
					New += ";";
					i += 2;
				}
			}
			else
			{
				// remove the '\n'
				i++;
			}
		}
		else
			New += quoted[i];
	}
	return New;
}
